//***************************************************************************//
//*                                                                         *//
//* Unit:    drj_Resources                                                  *//
//* Author:  Dark Magus                                                     *//
//* Date:    28.12.01                                                       *//
//*                                                                         *//
//*           XML-- *//
//*  (   )   Dream Journal "Wizardry".      *//
//*                                                                         *//
//***************************************************************************//
unit drj_FormRenderer;

//-[ INTERFACE ]---------------------------------------------------------------
interface

uses
  controls, stdctrls, sysutils, classes, extctrls,

  xml_Node;

const
  TAG_EDIT              : String = 'edit';
  TAG_GROUP             : String = 'group';
  TAG_RADIO             : String = 'radio';
  TAG_BUTTON            : String = 'button';
  TAG_OPTION            : String = 'option';
  TAG_SELECT            : String = 'select';
  TAG_STATIC            : String = 'static';
  TAG_CHECKBOX          : String = 'checkbox';
  TAG_TEXTAREA          : String = 'textarea';
  TAG_SEPARATOR         : String = 'separator';

const
  ATTRIBUTE_ID          : String = 'id';
  ATTRIBUTE_TOP         : String = 'top';
  ATTRIBUTE_HINT        : String = 'hint';
  ATTRIBUTE_LEFT        : String = 'left';
  ATTRIBUTE_WIDTH       : String = 'width';
  ATTRIBUTE_VALUE       : String = 'value';
  ATTRIBUTE_ACTION      : String = 'action';
  ATTRIBUTE_HEIGHT      : String = 'height';
  ATTRIBUTE_CHECKED     : String = 'checked';
  ATTRIBUTE_ORIENTATION : String = 'orientation';

const
  UNDEFINED_VALUE       : String = 'undefined';
  VERTICAL_VALUE        : String = 'vertical';
  HORIZONTAL_VALUE      : String = 'horizontal';
  TRUE_VALUE            : String = 'true';
  FALSE_VALUE           : String = 'false';
  UNDERLINE             : String = '_';

const
  EDIT_NAME             : String = 'EDITBOX';
  LIST_NAME             : String = 'LIST';
  MEMO_NAME             : String = 'MEMO';
  GROUP_NAME            : String = 'GROUPBOX';
  BUTTON_NAME           : String = 'BUTTON';
  STATIC_NAME           : String = 'STATIC';
  CHECKBOX_NAME         : String = 'CHECKBOX';
  RADIOBOX_NAME         : String = 'RADIOBOX';
  SEPARATOR_NAME        : String = 'BEVEL';

const
  ACTION_CLOSE        : String = 'close';
  ACTION_OK           : String = 'ok';
  ACTION_CANCEL       : String = 'cancel';

type TFormRenderer = class (TObject)
  private
    FEditCounter         : Integer;
    FListCounter         : Integer;
    FMemoCounter         : Integer;
    FGroupCounter        : Integer;
    FButtonCounter       : Integer;
    FStaticCounter       : Integer;
    FCheckboxCounter     : Integer;
    FRadioboxCounter     : Integer;
    FSeparatorCounter    : Integer;

    FCloseFormProcedure  : TNotifyEvent;
    FCancelFormProcedure : TNotifyEvent;

    function CreateEdit (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TEdit;
    function CreateMemo (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TMemo;
    function CreateGroup (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TGroupBox;
    function CreateButton (iLeft, iTop, iWidth, iHeight, iValue, iName, iAction, iHint : String; iContainer : TWinControl) : TButton;
    function CreateStatic (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TLabel;
    function CreateSelect (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TComboBox;
    function CreateCheckbox (iLeft, iTop, iWidth, iHeight, iValue, iName, iChecked, iHint : String; iContainer : TWinControl) : TCheckbox;
    function CreateRadiobox (iLeft, iTop, iWidth, iHeight, iValue, iName, iChecked, iHint : String; iContainer : TWinControl) : TRadioButton;
    function CreateSeparator (iLeft, iTop, iWidth, iHeight, iOrientation, iName : String; iContainer : TWinControl) : TBevel;

  public
    constructor Create;
    destructor Destroy; override;

    procedure RenderForm (iDescription : TXMLNode; iContainer : TWinControl);

  published
    property CloseFormProcedure  : TNotifyEvent write FCloseFormProcedure;
    property CancelFormProcedure : TNotifyEvent write FCancelFormProcedure;

end;

//-[ IMPLEMENTATION ]----------------------------------------------------------
implementation

constructor TFormRenderer.Create;
begin
  inherited Create;

  FEditCounter := 0;
  FListCounter := 0;
  FMemoCounter := 0;
  FGroupCounter := 0;
  FButtonCounter := 0;
  FStaticCounter := 0;
  FSeparatorCounter := 0;
end;

destructor TFormRenderer.Destroy;
begin
  inherited Destroy;
end;

procedure TFormRenderer.RenderForm (iDescription : TXMLNode; iContainer : TWinControl);
var
  i          : Integer;
  tempString : String;
  xmlNode    : TXMLNode;

  groupBox   : TGroupBox;
  comboBox   : TComboBox;
begin
  for i := 0 to iDescription.GetNumberOfChildren - 1 do
  begin
    xmlNode := iDescription.GetChild (i);

    //*** EDIT ***//
    if (xmlNode.TagName = TAG_EDIT) then
    begin
      CreateEdit (xmlNode.GetAttributeValue (ATTRIBUTE_LEFT),
                  xmlNode.GetAttributeValue (ATTRIBUTE_TOP),
                  xmlNode.GetAttributeValue (ATTRIBUTE_WIDTH),
                  xmlNode.GetAttributeValue (ATTRIBUTE_HEIGHT),
                  xmlNode.GetAttributeValue (ATTRIBUTE_VALUE),
                  xmlNode.GetAttributeValue (ATTRIBUTE_ID),
                  xmlNode.GetAttributeValue (ATTRIBUTE_HINT),
                  iContainer);
      Continue;
    end;

    //*** GROUP ***//
    if (xmlNode.TagName = TAG_GROUP) then
    begin
      groupBox := CreateGroup (xmlNode.GetAttributeValue (ATTRIBUTE_LEFT),
                               xmlNode.GetAttributeValue (ATTRIBUTE_TOP),
                               xmlNode.GetAttributeValue (ATTRIBUTE_WIDTH),
                               xmlNode.GetAttributeValue (ATTRIBUTE_HEIGHT),
                               xmlNode.GetAttributeValue (ATTRIBUTE_VALUE),
                               xmlNode.GetAttributeValue (ATTRIBUTE_ID),
                               xmlNode.GetAttributeValue (ATTRIBUTE_HINT),
                               iContainer);
      RenderForm (xmlNode, groupBox);
      Continue;
    end;

    //*** BUTTON ***//
    if (xmlNode.TagName = TAG_BUTTON) then
    begin
      CreateButton (xmlNode.GetAttributeValue (ATTRIBUTE_LEFT),
                    xmlNode.GetAttributeValue (ATTRIBUTE_TOP),
                    xmlNode.GetAttributeValue (ATTRIBUTE_WIDTH),
                    xmlNode.GetAttributeValue (ATTRIBUTE_HEIGHT),
                    xmlNode.GetAttributeValue (ATTRIBUTE_VALUE),
                    xmlNode.GetAttributeValue (ATTRIBUTE_ID),
                    xmlNode.GetAttributeValue (ATTRIBUTE_ACTION),
                    xmlNode.GetAttributeValue (ATTRIBUTE_HINT),
                    iContainer);
      Continue;
    end;

    //*** OPTION ***//
    if (xmlNode.TagName = TAG_OPTION) then
    begin
      tempString := xmlNode.GetAttributeValue (ATTRIBUTE_VALUE);
      if (tempString <> UNDEFINED_VALUE) then
      begin
        comboBox := (iContainer as TComboBox);
        comboBox.Items.Add (tempString);
      end;
      Continue;
    end;

    //*** SELECT ***//
    if (xmlNode.TagName = TAG_SELECT) then
    begin
      comboBox := CreateSelect (xmlNode.GetAttributeValue (ATTRIBUTE_LEFT),
                                xmlNode.GetAttributeValue (ATTRIBUTE_TOP),
                                xmlNode.GetAttributeValue (ATTRIBUTE_WIDTH),
                                xmlNode.GetAttributeValue (ATTRIBUTE_HEIGHT),
                                xmlNode.GetAttributeValue (ATTRIBUTE_VALUE),
                                xmlNode.GetAttributeValue (ATTRIBUTE_ID),
                                xmlNode.GetAttributeValue (ATTRIBUTE_HINT),
                                iContainer);
      RenderForm (xmlNode, comboBox);
      Continue;
    end;

    //*** STATIC ***//
    if (xmlNode.TagName = TAG_STATIC) then
    begin
      CreateStatic (xmlNode.GetAttributeValue (ATTRIBUTE_LEFT),
                    xmlNode.GetAttributeValue (ATTRIBUTE_TOP),
                    xmlNode.GetAttributeValue (ATTRIBUTE_WIDTH),
                    xmlNode.GetAttributeValue (ATTRIBUTE_HEIGHT),
                    xmlNode.GetAttributeValue (ATTRIBUTE_VALUE),
                    xmlNode.GetAttributeValue (ATTRIBUTE_ID),
                    xmlNode.GetAttributeValue (ATTRIBUTE_HINT),
                    iContainer);
      Continue;
    end;

    //*** TEXTAREA ***//
    if (xmlNode.TagName = TAG_TEXTAREA) then
    begin
      CreateMemo (xmlNode.GetAttributeValue (ATTRIBUTE_LEFT),
                  xmlNode.GetAttributeValue (ATTRIBUTE_TOP),
                  xmlNode.GetAttributeValue (ATTRIBUTE_WIDTH),
                  xmlNode.GetAttributeValue (ATTRIBUTE_HEIGHT),
                  xmlNode.GetAttributeValue (ATTRIBUTE_VALUE),
                  xmlNode.GetAttributeValue (ATTRIBUTE_ID),
                  xmlNode.GetAttributeValue (ATTRIBUTE_HINT),
                  iContainer);
      Continue;
    end;

    //*** CHECKBOX ***//
    if (xmlNode.TagName = TAG_CHECKBOX) then
    begin
      CreateCheckbox (xmlNode.GetAttributeValue (ATTRIBUTE_LEFT),
                      xmlNode.GetAttributeValue (ATTRIBUTE_TOP),
                      xmlNode.GetAttributeValue (ATTRIBUTE_WIDTH),
                      xmlNode.GetAttributeValue (ATTRIBUTE_HEIGHT),
                      xmlNode.GetAttributeValue (ATTRIBUTE_VALUE),
                      xmlNode.GetAttributeValue (ATTRIBUTE_ID),
                      xmlNode.GetAttributeValue (ATTRIBUTE_CHECKED),
                      xmlNode.GetAttributeValue (ATTRIBUTE_HINT),
                      iContainer);
      Continue;
    end;

    //*** RADIOBUTTON ***//
    if (xmlNode.TagName = TAG_RADIO) then
    begin
      CreateRadiobox (xmlNode.GetAttributeValue (ATTRIBUTE_LEFT),
                      xmlNode.GetAttributeValue (ATTRIBUTE_TOP),
                      xmlNode.GetAttributeValue (ATTRIBUTE_WIDTH),
                      xmlNode.GetAttributeValue (ATTRIBUTE_HEIGHT),
                      xmlNode.GetAttributeValue (ATTRIBUTE_VALUE),
                      xmlNode.GetAttributeValue (ATTRIBUTE_ID),
                      xmlNode.GetAttributeValue (ATTRIBUTE_CHECKED),
                      xmlNode.GetAttributeValue (ATTRIBUTE_HINT),
                      iContainer);
      Continue;
    end;

    //*** SEPARATOR ***//
    if (xmlNode.TagName = TAG_SEPARATOR) then
    begin
      CreateSeparator (xmlNode.GetAttributeValue (ATTRIBUTE_LEFT),
                       xmlNode.GetAttributeValue (ATTRIBUTE_TOP),
                       xmlNode.GetAttributeValue (ATTRIBUTE_WIDTH),
                       xmlNode.GetAttributeValue (ATTRIBUTE_HEIGHT),
                       xmlNode.GetAttributeValue (ATTRIBUTE_ORIENTATION),
                       xmlNode.GetAttributeValue (ATTRIBUTE_ID),
                       iContainer);
      Continue;
    end;
  end;
end;

function TFormRenderer.CreateEdit (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TEdit;
var
  component : TEdit;
begin
  component := TEdit.Create (iContainer);
  component.Ctl3D := False;
  component.Parent := iContainer;

  if (iLeft <> UNDEFINED_VALUE) then
  begin
    component.Left := StrToInt (iLeft);
  end;
  if (iTop <> UNDEFINED_VALUE) then
  begin
    component.Top := StrToInt (iTop);
  end;
  if (iWidth <> UNDEFINED_VALUE) then
  begin
    component.Width := StrToInt (iWidth);
  end;
  if (iHeight <> UNDEFINED_VALUE) then
  begin
    component.Height := StrToInt (iHeight);
  end;
  if (iName <> UNDEFINED_VALUE) then
  begin
    component.Name := iName;
  end
  else
  begin
    component.Name := EDIT_NAME + UNDERLINE + IntToStr (FEditCounter);
    FEditCounter := FEditCounter + 1;
  end;
  if (iValue <> UNDEFINED_VALUE) then
  begin
    component.Text := iValue;
  end
  else
  begin
    component.Text := '';
  end;
  if (iHint <> UNDEFINED_VALUE) then
  begin
    component.Hint := iHint;
    component.ShowHint := True;
  end;
  Result := component;
end;

function TFormRenderer.CreateGroup (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TGroupBox;
var
  component : TGroupBox;
begin
  component := TGroupBox.Create (iContainer);
  component.Ctl3D := False;
  component.Parent := iContainer;

  if (iLeft <> UNDEFINED_VALUE) then
  begin
    component.Left := StrToInt (iLeft);
  end;
  if (iTop <> UNDEFINED_VALUE) then
  begin
    component.Top := StrToInt (iTop);
  end;
  if (iWidth <> UNDEFINED_VALUE) then
  begin
    component.Width := StrToInt (iWidth);
  end;
  if (iHeight <> UNDEFINED_VALUE) then
  begin
    component.Height := StrToInt (iHeight);
  end;
  if (iName <> UNDEFINED_VALUE) then
  begin
    component.Name := iName;
  end
  else
  begin
    component.Name := GROUP_NAME + UNDERLINE + IntToStr (FGroupCounter);
    FGroupCounter := FGroupCounter + 1;
  end;
  if (iValue <> UNDEFINED_VALUE) then
  begin
    component.Caption := iValue;
  end
  else
  begin
    component.Caption := '';
  end;
  if (iHint <> UNDEFINED_VALUE) then
  begin
    component.Hint := iHint;
    component.ShowHint := True;
  end;
  Result := component;
end;

function TFormRenderer.CreateButton (iLeft, iTop, iWidth, iHeight, iValue, iName, iAction, iHint : String; iContainer : TWinControl) : TButton;
var
  component : TButton;
begin
  component := TButton.Create (iContainer);
  component.Parent := iContainer;

  if (iLeft <> UNDEFINED_VALUE) then
  begin
    component.Left := StrToInt (iLeft);
  end;
  if (iTop <> UNDEFINED_VALUE) then
  begin
    component.Top := StrToInt (iTop);
  end;
  if (iWidth <> UNDEFINED_VALUE) then
  begin
    component.Width := StrToInt (iWidth);
  end;
  if (iHeight <> UNDEFINED_VALUE) then
  begin
    component.Height := StrToInt (iHeight);
  end;
  if (iName <> UNDEFINED_VALUE) then
  begin
    component.Name := iName;
  end
  else
  begin
    component.Name := BUTTON_NAME + UNDERLINE + IntToStr (FButtonCounter);
    FButtonCounter := FButtonCounter + 1;
  end;
  if (iValue <> UNDEFINED_VALUE) then
  begin
    component.Caption := iValue;
  end
  else
  begin
    component.Caption := '';
  end;

  if ((iAction = ACTION_CLOSE) or (iAction = ACTION_OK)) then
  begin
    component.OnClick := FCloseFormProcedure;
  end;
  if (iAction = ACTION_CANCEL) then
  begin
    component.OnClick := FCancelFormProcedure;
  end;

  if (iHint <> UNDEFINED_VALUE) then
  begin
    component.Hint := iHint;
    component.ShowHint := True;
  end;
  Result := component;
end;

function TFormRenderer.CreateStatic (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TLabel;
var
  component : TLabel;
begin
  component := TLabel.Create (iContainer);
  component.AutoSize := False;
  component.WordWrap := False;
  component.Parent := iContainer;

  if (iLeft <> UNDEFINED_VALUE) then
  begin
    component.Left := StrToInt (iLeft);
  end;
  if (iTop <> UNDEFINED_VALUE) then
  begin
    component.Top := StrToInt (iTop);
  end;
  if (iWidth <> UNDEFINED_VALUE) then
  begin
    component.Width := StrToInt (iWidth);
  end;
  if (iHeight <> UNDEFINED_VALUE) then
  begin
    component.Height := StrToInt (iHeight);
  end;
  if (iName <> UNDEFINED_VALUE) then
  begin
    component.Name := iName;
  end
  else
  begin
    component.Name := STATIC_NAME + UNDERLINE + IntToStr (FStaticCounter);
    FStaticCounter := FStaticCounter + 1;
  end;
  if (iValue <> UNDEFINED_VALUE) then
  begin
    component.Caption := iValue;
  end
  else
  begin
    component.Caption := '';
  end;
  if (iHint <> UNDEFINED_VALUE) then
  begin
    component.Hint := iHint;
    component.ShowHint := True;
  end;
  Result := component;
end;

function TFormRenderer.CreateSelect (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TComboBox;
var
  component : TComboBox;
begin
  component := TComboBox.Create (iContainer);
  component.Style := csDropDownList;
  component.Text := '';
  component.Ctl3D := False;
  component.Parent := iContainer;

  if (iLeft <> UNDEFINED_VALUE) then
  begin
    component.Left := StrToInt (iLeft);
  end;
  if (iTop <> UNDEFINED_VALUE) then
  begin
    component.Top := StrToInt (iTop);
  end;
  if (iWidth <> UNDEFINED_VALUE) then
  begin
    component.Width := StrToInt (iWidth);
  end;
  if (iHeight <> UNDEFINED_VALUE) then
  begin
    component.Height := StrToInt (iHeight);
  end;
  if (iValue <> UNDEFINED_VALUE) then
  begin
    component.Text := iValue;
  end;
  if (iName <> UNDEFINED_VALUE) then
  begin
    component.Name := iName;
  end
  else
  begin
    component.Name := LIST_NAME + UNDERLINE + IntToStr (FListCounter);
    FListCounter := FListCounter + 1;
  end;
  if (iHint <> UNDEFINED_VALUE) then
  begin
    component.Hint := iHint;
    component.ShowHint := True;
  end;
  Result := component;
end;

function TFormRenderer.CreateMemo (iLeft, iTop, iWidth, iHeight, iValue, iName, iHint : String; iContainer : TWinControl) : TMemo;
var
  component : TMemo;
begin
  component := TMemo.Create (iContainer);
  component.ScrollBars := ssVertical; 
  component.Parent := iContainer;

  if (iLeft <> UNDEFINED_VALUE) then
  begin
    component.Left := StrToInt (iLeft);
  end;
  if (iTop <> UNDEFINED_VALUE) then
  begin
    component.Top := StrToInt (iTop);
  end;
  if (iWidth <> UNDEFINED_VALUE) then
  begin
    component.Width := StrToInt (iWidth);
  end;
  if (iHeight <> UNDEFINED_VALUE) then
  begin
    component.Height := StrToInt (iHeight);
  end;
  if (iName <> UNDEFINED_VALUE) then
  begin
    component.Name := iName;
  end
  else
  begin
    component.Name := MEMO_NAME + UNDERLINE + IntToStr (FMemoCounter);
    FMemoCounter := FMemoCounter + 1;
  end;
  if (iValue <> UNDEFINED_VALUE) then
  begin
    component.Text := iValue;
  end
  else
  begin
    component.Text := '';
  end;
  if (iHint <> UNDEFINED_VALUE) then
  begin
    component.Hint := iHint;
    component.ShowHint := True;
  end;
  Result := component;
end;

function TFormRenderer.CreateSeparator (iLeft, iTop, iWidth, iHeight, iOrientation, iName : String; iContainer : TWinControl) : TBevel;
var
  component : TBevel;
begin
  component := TBevel.Create (iContainer);
  component.Parent := iContainer;

  if (iLeft <> UNDEFINED_VALUE) then
  begin
    component.Left := StrToInt (iLeft);
  end;
  if (iTop <> UNDEFINED_VALUE) then
  begin
    component.Top := StrToInt (iTop);
  end;
  if (iWidth <> UNDEFINED_VALUE) then
  begin
    component.Width := StrToInt (iWidth);
  end;
  if (iHeight <> UNDEFINED_VALUE) then
  begin
    component.Height := StrToInt (iHeight);
  end;
  if (iOrientation <> UNDEFINED_VALUE) then
  begin
    if (iOrientation = VERTICAL_VALUE) then
    begin
      component.Shape := bsLeftLine;
    end;
    if (iOrientation = HORIZONTAL_VALUE) then
    begin
      component.Shape := bsTopLine;
    end;
  end;
  if (iName <> UNDEFINED_VALUE) then
  begin
    component.Name := iName;
  end
  else
  begin
    component.Name := SEPARATOR_NAME + UNDERLINE + IntToStr (FSeparatorCounter);
    FSeparatorCounter := FSeparatorCounter + 1;
  end;
  Result := component;
end;

function TFormRenderer.CreateCheckbox (iLeft, iTop, iWidth, iHeight, iValue, iName, iChecked, iHint : String; iContainer : TWinControl) : TCheckbox;
var
  component : TCheckbox;
begin
  component := TCheckbox.Create (iContainer);
  component.Ctl3D := False;
  component.Parent := iContainer;

  if (iLeft <> UNDEFINED_VALUE) then
  begin
    component.Left := StrToInt (iLeft);
  end;
  if (iTop <> UNDEFINED_VALUE) then
  begin
    component.Top := StrToInt (iTop);
  end;
  if (iWidth <> UNDEFINED_VALUE) then
  begin
    component.Width := StrToInt (iWidth);
  end;
  if (iHeight <> UNDEFINED_VALUE) then
  begin
    component.Height := StrToInt (iHeight);
  end;
  if (iName <> UNDEFINED_VALUE) then
  begin
    component.Name := iName;
  end
  else
  begin
    component.Name := CHECKBOX_NAME + UNDERLINE + IntToStr (FCheckboxCounter);
    FCheckboxCounter := FCheckboxCounter + 1;
  end;
  if (iValue <> UNDEFINED_VALUE) then
  begin
    component.Caption := iValue;
  end
  else
  begin
    component.Caption := '';
  end;
  if (iChecked <> UNDEFINED_VALUE) then
  begin
    component.Checked := (iChecked = TRUE_VALUE);
  end;
  if (iHint <> UNDEFINED_VALUE) then
  begin
    component.Hint := iHint;
    component.ShowHint := True;
  end;
  Result := component;
end;

function TFormRenderer.CreateRadiobox (iLeft, iTop, iWidth, iHeight, iValue, iName, iChecked, iHint : String; iContainer : TWinControl) : TRadioButton;
var
  component : TRadioButton;
begin
  component := TRadioButton.Create (iContainer);
  component.Ctl3D := False;
  component.Parent := iContainer;

  if (iLeft <> UNDEFINED_VALUE) then
  begin
    component.Left := StrToInt (iLeft);
  end;
  if (iTop <> UNDEFINED_VALUE) then
  begin
    component.Top := StrToInt (iTop);
  end;
  if (iWidth <> UNDEFINED_VALUE) then
  begin
    component.Width := StrToInt (iWidth);
  end;
  if (iHeight <> UNDEFINED_VALUE) then
  begin
    component.Height := StrToInt (iHeight);
  end;
  if (iName <> UNDEFINED_VALUE) then
  begin
    component.Name := iName;
  end
  else
  begin
    component.Name := RADIOBOX_NAME + UNDERLINE + IntToStr (FRadioboxCounter);
    FRadioboxCounter := FRadioboxCounter + 1;
  end;
  if (iValue <> UNDEFINED_VALUE) then
  begin
    component.Caption := iValue;
  end
  else
  begin
    component.Caption := '';
  end;
  if (iChecked <> UNDEFINED_VALUE) then
  begin
    component.Checked := (iChecked = TRUE_VALUE);
  end;
  if (iHint <> UNDEFINED_VALUE) then
  begin
    component.Hint := iHint;
    component.ShowHint := True;
  end;
  Result := component;
end;

//-[ END ]---------------------------------------------------------------------
end.
