//***************************************************************************//
//*                                                                         *//
//* Unit:    drj_GraphPainter                                               *//
//* Author:  Colonel                                                        *//
//*          Dark Magus                                                     *//
//* Date:    15.10.01                                                       *//
//*                                                                         *//
//*              *//
//*  Dream Journal (v. 2.0).                                         *//
//*                                                                         *//
//***************************************************************************//


unit drj_GraphPainter;

//-[ INTERFACE ]---------------------------------------------------------------
interface

uses
  Windows, Graphics, Classes, extctrls, Sysutils,

  colonel_Utility;

const
  maxLabelsCount = 1000;

type TDJGraphType  = (gtDot,
                      gtRDot,
                      gtLinearDot,
                      gtLinearRDot,
                      gtLinear,
                      gtHHistogram,
                      gtVHistogram);

type TDJGraphValue = record
  name : String;
  x    : Double;
  y    : Double;
end;

type TDJGraphValueArray = array of TDJGraphValue;

type TDJGraphHeader = record
        caption   : string;
        Font      : TFont;
        Alignment : TAlignment;
      end;

type TDJGraph = class (TObject)
  private
    fMinX         : double;           // Min      .
    fMaxX         : double;           // Max      .
    fMinY         : double;           // Min      .
    fMaxY         : double;           // Max       .
    fHistColWidth : double;           //     %    / 100
                                      // .. 0.13 = 13%
    fValues : TDJGraphValueArray;     //    .
    fGType  : TDJGraphType;           //  

    fAxisColor         : TColor;      //   .
    fValuesColor       : TColorArray; //   .
    fValuesBorderColor : TColor;      //   .
    fBgColor           : TColor;      //  .
    fPen               : TPen;        //  Linear  LinearDot

    fXLabelsFrom       : double;      //     X
    fXLabelsStep       : double;      //     
    fXLabelsCount      : integer;     // -   (0 -  , -1 -  )

    fYLabelsFrom       : double;      //     Y
    fYLabelsStep       : double;      //     
    fYLabelsCount      : integer;     // -   (0 -  , -1 -  )

    fXLabelsFormat     : string;      //      
    fYLabelsFormat     : string;      //      Y

    fLabelsFont        : TFont;       //    

    fLBitmap           : TBitmap;     //  ,    

    fDrawArea          : TRect;       //     .


    procedure pSetValues (iAValues : TDJGraphValueArray);

    procedure pDrawHorizontalHistogram;
    procedure pDrawDotHistogram;
    procedure pDrawLinearHistogram;
    procedure pDrawDotLinearHistogram;
    procedure pDrawVerticalHistogram;

    function  pDrawHAxis   : TRect;
    function  pDrawVAxis   : TRect;
    function  pDrawHeaders : TRect;


  public
    Header : TDJGraphHeader;
    Footer : TDJGraphHeader;

    constructor Create;
    destructor Destroy; override;

    //   
    procedure AddValue (iName: string; iX, iY : double);

    //   
    procedure ClearValues;

    //      (   ,
    //    )
    procedure SetValuesColor (const iAColor : array of TColor);

    // PROPERTIES
    //   
    property minX : double read fMinX write fMinX;
    property maxX : double read fMaxX write fMaxX;
    property minY : double read fMinY write fMinY;
    property maxY : double read fMaxY write fMaxY;
    //   :  gtDot-gtVHistogram
    property GType : TDJGraphType   read fGType write fGType;

    // ------   : ------
    //      -   
    property XLabelsFrom  : double  read fXLabelsFrom   write fXLabelsFrom;
    //      -    
    property XLabelsStep  : double  read fXLabelsStep   write fXLabelsStep;
    //      - -   ( = -1,    )
    property XLabelsCount : integer read fXLabelsCount  write fXLabelsCount;
    //      -    (  Delphi FORMAT)
    property XLabelsFormat: string  read fXLabelsFormat write fXLabelsFormat;
    // ------   Y: ------
    property YLabelsFrom  : double  read fYLabelsFrom   write fYLabelsFrom;
    property YLabelsStep  : double  read fYLabelsStep   write fYLabelsStep;
    property YLabelsCount : integer read fYLabelsCount  write fYLabelsCount;
    property YLabelsFormat: string  read fYLabelsFormat write fYLabelsFormat;
    //    
    property LabelsFont   : TFont   read fLabelsFont;
    //   X,Y
    property AxisColor         : TColor read fAxisColor write fAxisColor;
    //     %     (0.05 = 5%)
    property HistColWidth : double read fHistColWidth write fHistColWidth;
    // ,    gtDot  gtLinearDot
    property Pen : TPen read fPen;
    //   
    property Values       : TDJGraphValueArray read fValues write pSetValues;
    //     ( clNone -  )
    property ValuesBorderColor : TColor read fValuesBorderColor write fValuesBorderColor;
    //     ( clNone -  )
    property BgColor           : TColor read fBgColor write fBgColor;

    //  
    procedure Paint (where : TBitmap; Area: TRect);

end;

//-[ IMPLEMENTATION ]----------------------------------------------------------
implementation

//------------------------------------------------------------------------------
//                          CONSTRUCTORS & DESTRUCTORS
//------------------------------------------------------------------------------
constructor TDJGraph.create;
begin
  inherited create;

  fMinX    := 1;
  fMaxX    := 10;
  fMinY    := 0;
  fMaxY    := 10;
  fHistColWidth  := 0.05;

  fGType        := gtHHistogram;

  fXLabelsFrom   := fMinX;
  fXLabelsStep   := 1;
  fXLabelsCount  := -1;
  fXLabelsFormat := '%.0n';

  fYLabelsFrom   := fMinY;
  fYLabelsStep   := 1;
  fYLabelsCount  := -1;
  fYLabelsFormat := '%.0n';

  fLabelsFont := TFont.Create;
  fPen        := TPen.Create;
  fPen.Style  := psDot;
  fPen.Color  := clBlack;
  fPen.Width  := 1;

  Header.caption   := '';
  Header.Font      := TFont.Create;
  Header.Alignment := taCenter;
  Footer.caption   := '';
  Footer.Font      := TFont.Create;
  Footer.Alignment := taCenter;

  setLength (fValues,0);
  fAxisColor := clBlack;
  fBgColor   := clNone;
  SetValuesColor ([clRed]);
  fValuesBorderColor := clNone;

  fDrawArea := Rect (0,0,0,0);
end;

destructor TDJGraph.destroy;
begin
  fLabelsFont.free;
  fPen.free;
  inherited destroy;
end;

//------------------------------------------------------------------------------
//                            PROTECTED
//------------------------------------------------------------------------------
procedure TDJGraph.pSetValues (iAValues : TDJGraphValueArray);
var
  i : integer;
begin
  setlength (fValues, length (iAValues));
  for i := 0 to High (iAValues) do
  begin
    fValues [i] := iAValues[i];
  end;
end;

procedure TDJGraph.SetValuesColor (const iAColor : Array of TColor);
var
  i : integer;
begin
  setlength (fValuesColor, length (iAColor));
  for i := 0 to High (iAColor) do
  begin
    fValuesColor [i] := iAColor[i];
  end;
end;

//------------------------------------------------------------------------------
//                             
//------------------------------------------------------------------------------

  function  TDJGraph.pDrawHeaders : TRect;
  var bXWidth     : integer;
      bYHeight    : integer;
      bXValue     : integer;
      bRect       : TRect;
  begin
    bRect := fDrawArea;

    with bRect do
      if fBgColor<>clNone then
         iFillBox (fLBitmap, left,top,right-left,bottom-top,fBgColor,fBgColor);

    if trim(Header.caption)<>'' then
    begin
      fLBitmap.Canvas.Font.Assign (Header.font);
      bXWidth := fLBitmap.Canvas.TextWidth  (Header.caption);
      bYHeight:= fLBitmap.Canvas.TextHeight (Header.caption);
      bXValue := 0;
      case Header.Alignment of
        taLeftJustify  : bXValue := 0;
        taRightJustify : bXValue := round  (bRect.right-bRect.Left-bXWidth);
        taCenter       : bXValue := round ((bRect.Right-bRect.Left-bXWidth)/2);
      end;
      fLBitmap.Canvas.TextOut (bRect.left + bXValue, bRect.Top, Header.caption);
      bRect.Top := bRect.Top + bYHeight + 10;
    end;

    if trim(Footer.caption)<>'' then
    begin
      fLBitmap.Canvas.Font.Assign (Footer.font);
      bXWidth := fLBitmap.Canvas.TextWidth  (Footer.caption);
      bYHeight:= fLBitmap.Canvas.TextHeight (Footer.caption);
      bXValue := 0;
      case Footer.Alignment of
        taLeftJustify  : bXValue := 0;
        taRightJustify : bXValue := round  (bRect.right-bRect.Left-bXWidth);
        taCenter       : bXValue := round ((bRect.Right-bRect.Left-bXWidth)/2);
      end;
      fLBitmap.Canvas.TextOut (bRect.left + bXValue, bRect.Bottom - bYHeight, Footer.caption);
      bRect.Bottom := bRect.Bottom - bYHeight - 10;
    end;
    result := bRect;
  end;


function TDJGraph.pDrawVAxis : TRect;
var
  bHeight          : Integer;
  bWidth           : integer;  //   
  bXWidth          : integer;  //  
  bYHeight         : integer;  //  
  i                : integer;
  bYValue          : integer;
  bXValue          : integer;
  bRect            : TRect;
  txtLabel         : string;
  maxLabelsCounter : integer;
  bLabelPos        : double;
  bHWidth          : integer;
  tmpMaxSize       : integer;
begin
  fLBitmap.Canvas.Pen.Style := psSolid;
  fLBitmap.Canvas.Pen.Width := 1;

  bRect := pDrawHeaders;
  //----------------------------------------------------------------------------
  //                      
  //----------------------------------------------------------------------------
  fLBitmap.Canvas.Font.Assign (fLabelsFont);

  maxLabelsCounter := fXLabelsCount;
  bLabelPos := fXLabelsFrom;
  if (fXLabelsCount <> 0) then
  begin
    if ((maxLabelsCounter > maxLabelsCount) or (maxLabelsCounter < 0)) then
    begin
      maxLabelsCounter := maxLabelsCount;
    end;
    //     
    tmpMaxSize := 0;
    for i := 1 to maxLabelsCounter do
    begin
      if (bLabelPos > fMaxX) then
      begin
        Break;
      end
      else
      begin
        if ((bLabelPos >= fMinX) and (bLabelPos <= fMaxX)) then
        begin
          txtLabel := format (fXLabelsFormat, [bLabelPos]);
          bYHeight := fLBitmap.Canvas.TextHeight (txtLabel);
          if (tmpMaxSize < bYHeight) then
          begin
            tmpMaxSize := bYHeight;
          end;
        end;
      end;
      bLabelPos := bLabelPos + fXLabelsStep;
    end;
    if (tmpMaxSize > 0) then
    begin
      bRect.Bottom := bRect.Bottom - tmpMaxSize - 3;
    end;
  end;

  //----------------------------------------------------------------------------
  //                      
  //----------------------------------------------------------------------------
    maxLabelsCounter := fYLabelsCount;
    bLabelPos := fYLabelsFrom;
    if fYLabelsCount<>0 then
    begin
      if (maxLabelsCounter>maxLabelsCount) or
         (maxLabelsCounter<0) then maxLabelsCounter := maxLabelsCount;
      tmpMaxSize := 0;
      for i := 1 to maxLabelsCounter do
      begin
        if   bLabelPos>fMaxY then break else
        if (bLabelPos>=fMinY) and (bLabelPos<=fMaxY) then
        begin
          txtLabel := format  (fYLabelsFormat,[bLabelPos]);
          bXWidth  := fLBitmap.Canvas.TextWidth (txtLabel);
          if tmpMaxSize<bXWidth then tmpMaxSize := bXWidth;
        end;
        bLabelPos := bLabelPos + fXLabelsStep;
      end;
      if tmpMaxSize>0 then bRect.Left := bRect.Left + tmpMaxSize + 4;
    end;
//---------------------------------------------------------------------------
//                      X
//---------------------------------------------------------------------------
    with bRect do
    begin
      bHeight := Bottom - Top;
      bWidth  := Right - Left;
    end;

    if fGType=gtLinear then
         bHWidth := 0
    else bHWidth := round(bWidth*fHistColWidth);

    maxLabelsCounter := fXLabelsCount;
    if fXLabelsCount<>0 then
    begin
      if (maxLabelsCounter>maxLabelsCount) or
         (maxLabelsCounter<0) then maxLabelsCounter := maxLabelsCount;
      bLabelPos := fXLabelsFrom;
      for i := 1 to maxLabelsCounter do
      begin
        if   bLabelPos>fMaxX then break else
        if (bLabelPos>=fMinX) and
           (bLabelPos<=fMaxX) then
        begin
          txtLabel := format  (fXLabelsFormat, [bLabelPos]);
          bXWidth  := fLBitmap.Canvas.TextWidth (txtLabel);
          bXValue  := bRect.Left + round ((bLabelPos - fMinX) / (fMaxX - fMinX) * (bWidth - bHWidth) + bHWidth / 2 - bXWidth / 2);
          bYValue  := bRect.Bottom + 3;
          fLBitmap.Canvas.TextOut (bXValue, bYValue, txtLabel);
          iLine (fLBitmap, bXValue + bXWidth / 2, bYValue - 3, bXValue + bXWidth / 2, bYValue, fAxisColor);
        end;
        bLabelPos := bLabelPos + fXLabelsStep;
      end;
    end;
//---------------------------------------------------------------------------
//                      Y
//---------------------------------------------------------------------------
    maxLabelsCounter := fYLabelsCount;
    if fYLabelsCount<>0 then
    begin
      if (maxLabelsCounter>maxLabelsCount) or
         (maxLabelsCounter<0) then maxLabelsCounter := maxLabelsCount;
      bLabelPos := fYLabelsFrom;
      for i := 1 to maxLabelsCounter do
      begin
        if   bLabelPos>fMaxY then break else
        if (bLabelPos>=fMinY) and
           (bLabelPos<=fMaxY) then
        begin
          txtLabel := format  (fYLabelsFormat,[bLabelPos]);
          bXWidth  := fLBitmap.Canvas.TextWidth  (txtLabel);
          bYHeight := fLBitmap.Canvas.TextHeight (txtLabel);
          bXValue  := -4 + bRect.Left - bXWidth;
          bYValue  := bRect.Bottom - round((bLabelPos-fMinY)/(fMaxY - fMinY)*bHeight + bYHeight/2);
          fLBitmap.Canvas.TextOut (bXValue,bYValue,txtLabel);
          iLine (fLBitmap, bRect.Left -2, bYValue+bYHeight/2, bRect.Left, bYValue+bYHeight/2, fAxisColor);
        end;
        bLabelPos := bLabelPos + fYLabelsStep;
      end;
    end;
// ---------------------------------------------------------------------------
//                                  
// ---------------------------------------------------------------------------
    with bRect do
    begin
      iLine (fLBitmap, Left, Bottom, Left, Top,     fAxisColor);
      iLine (fLBitmap, Left, Bottom, Right, Bottom, fAxisColor);
    end;

    result := bRect;
  end;


  function TDJGraph.pDrawHAxis:TRect;
  var bHeight, bWidth  : integer;
      bXWidth     : integer;
      bYHeight    : integer;
      i           : integer;
      bYValue     : integer;
      bXValue     : integer;
      bRect       : TRect;
      txtLabel    : string;
      maxLabelsCounter : integer;
      bLabelPos   : double;
      bHWidth     : integer;
      tmpMaxSize  : integer;
  begin
    fLBitmap.Canvas.Pen.Style := psSolid;
    fLBitmap.Canvas.Pen.Width := 1;

    bRect := pDrawHeaders;
//---------------------------------------------------------------------------
//                      
//---------------------------------------------------------------------------
    fLBitmap.Canvas.Font.Assign (fLabelsFont);

    maxLabelsCounter := fXLabelsCount;
    bLabelPos := fXLabelsFrom;
    if fXLabelsCount<>0 then
    begin
      if (maxLabelsCounter>maxLabelsCount) or
         (maxLabelsCounter<0) then maxLabelsCounter := maxLabelsCount;
        //     
      tmpMaxSize := 0;
      for i := 1 to maxLabelsCounter do
      begin
        if   bLabelPos>fMaxX then break else
        if (bLabelPos>=fMinX) and (bLabelPos<=fMaxX) then
        begin
          txtLabel := format  (fYLabelsFormat,[bLabelPos]);
          bXWidth  := fLBitmap.Canvas.TextHeight (txtLabel);
          if tmpMaxSize<bXWidth then tmpMaxSize := bXWidth;
        end;
        bLabelPos := bLabelPos + fXLabelsStep;
      end;
      if tmpMaxSize>0 then bRect.Bottom := bRect.Bottom - tmpMaxSize - 3;
    end;
//---------------------------------------------------------------------------
//                      
//---------------------------------------------------------------------------
    maxLabelsCounter := fYLabelsCount;
    bLabelPos := fYLabelsFrom;
    if fYLabelsCount<>0 then
    begin
      if (maxLabelsCounter>maxLabelsCount) or
         (maxLabelsCounter<0) then maxLabelsCounter := maxLabelsCount;
      tmpMaxSize := 0;
      for i := 1 to maxLabelsCounter do
      begin
        if   bLabelPos>fMaxY then break else
        if (bLabelPos>=fMinY) and (bLabelPos<=fMaxY) then
        begin
          txtLabel := format  (fXLabelsFormat,[bLabelPos]);
          bYHeight := fLBitmap.Canvas.TextWidth (txtLabel);
          if tmpMaxSize<bYHeight then tmpMaxSize := bYHeight;
        end;
        bLabelPos := bLabelPos + fXLabelsStep;
      end;
      if tmpMaxSize>0 then bRect.Left := bRect.Left + tmpMaxSize + 4;
    end;
//---------------------------------------------------------------------------
//                      X
//---------------------------------------------------------------------------
    with bRect do
    begin
      bHeight := Bottom - Top;
      bWidth  := Right - Left;
    end;

    bHWidth := round(bWidth*fHistColWidth);

    maxLabelsCounter := fYLabelsCount;
    if fYLabelsCount<>0 then
    begin
      if (maxLabelsCounter>maxLabelsCount) or
         (maxLabelsCounter<0) then maxLabelsCounter := maxLabelsCount;
      bLabelPos := fYLabelsFrom;
      for i := 1 to maxLabelsCounter do
      begin
        if  bLabelPos>fMaxY then break else
        if (bLabelPos>=fMinY) and
           (bLabelPos<=fMaxY) then
        begin
          txtLabel := format  (fYLabelsFormat,[bLabelPos]);
          bXWidth  := fLBitmap.Canvas.TextWidth  (txtLabel);
          bXValue  := bRect.Left   + round((bLabelPos-fMinY)/(fMaxY - fMinY)*bWidth - bXWidth/2);
          bYValue  := bRect.Bottom + 3;
          fLBitmap.Canvas.TextOut (bXValue,bYValue,txtLabel);
          iLine (fLBitmap, bXValue+bXWidth/2, bYValue-2, bXValue+bXWidth/2, bYValue, fAxisColor);
        end;
        bLabelPos := bLabelPos + fYLabelsStep;
      end;
    end;
//---------------------------------------------------------------------------
//                      Y
//---------------------------------------------------------------------------
    maxLabelsCounter := fXLabelsCount;
    if fXLabelsCount<>0 then
    begin
      if (maxLabelsCounter>maxLabelsCount) or
         (maxLabelsCounter<0) then maxLabelsCounter := maxLabelsCount;
      bLabelPos := fXLabelsFrom;
      for i := 1 to maxLabelsCounter do
      begin
        if   bLabelPos>fMaxX then break else
        if (bLabelPos>=fMinX) and
           (bLabelPos<=fMaxX) then
        begin
          txtLabel := format  (fXLabelsFormat,[bLabelPos]);
          bXWidth  := fLBitmap.Canvas.TextWidth  (txtLabel);
          bYHeight := fLBitmap.Canvas.TextHeight (txtLabel);
          bXValue  := -4 + bRect.Left - bXWidth;
          bYValue  := bRect.Bottom - round((bLabelPos-fMinX)/(fMaxX - fMinX)*(bHeight-bHWidth)+ bYHeight);
          fLBitmap.Canvas.TextOut (bXValue,bYValue,txtLabel);
          iLine (fLBitmap, bRect.Left - 2, bYValue+bYHeight/2, bRect.Left, bYValue+bYHeight/2, fAxisColor);
        end;
        bLabelPos := bLabelPos + fXLabelsStep;
      end;
    end;
// ---------------------------------------------------------------------------
//                                  
// ---------------------------------------------------------------------------
    with bRect do
    begin
      fLBitmap.Canvas.Pen.Style := psSolid;
      fLBitmap.Canvas.Pen.Width := 1;
      iLine (fLBitmap, Left, Bottom, Left, Top,     fAxisColor);
      iLine (fLBitmap, Left, Bottom, Right, Bottom, fAxisColor);
    end;

    result := bRect;
  end;




  procedure TDJGraph.pDrawVerticalHistogram;
  var bHeight, bWidth  : integer;  //   
      bXWidth     : integer;       //  
      i,nY,nX     : integer;
      bYValue     : integer;
      bXValue     : integer;
      BGR         : integer;
      bRect       : TRect;
  begin
// ---------------------------------------------------------------------------
//                 
// ---------------------------------------------------------------------------
    bRect := pDrawVAxis;

    bRect.Left   := bRect.Left   + 2;
    bRect.Bottom := bRect.Bottom - 2;

    with bRect do
    begin
      bHeight := Bottom - Top;
      bWidth  := Right - Left;
    end;
// ---------------------------------------------------------------------------
//                               
// ---------------------------------------------------------------------------
     //   
    bXWidth := round(bWidth*fHistColWidth);

    fLBitmap.Canvas.Pen.Style := psSolid;
    fLBitmap.Canvas.Pen.Width := 1;

    for i:=0 to high (fValues) do
      if (fValues[i].x>=fMinX) and (fValues[i].x<=fMaxX) then
      begin
          //  YMax  
        bYValue := round((fValues[i].y-fMinY)/(fMaxY - fMinY)*bHeight);
          //     
        if bYValue>bHeight then bYValue := bHeight
        else if bYValue<0 then bYValue := 0;
          //  X  
        bXValue := round((fValues[i].x-fMinX)/(fMaxX - fMinX)*(bWidth-bXWidth));
          //  
        for nY := 1 to bYValue - 1 do
        begin
          BGR := Color2BGR (GetColorFromList(fValuesColor, nY/bHeight));
          for nX := 0 to bXWidth - 1 do
            with bRect do
              qPutPixel (fLBitmap, Left+bXValue+nX, Bottom - nY,BGR);
        end;

        if (fValuesBorderColor<>clNone) and (bYValue>0) then
           with bRect do
            iRectangle (fLBitmap,
                        Left+bXValue,           Bottom-(bYValue),
                        Left+bXValue + bXWidth, Bottom, fValuesBorderColor);
      end;
  end;


  procedure TDJGraph.pDrawDotHistogram;
  var bHeight, bWidth  : integer;  //   
      bXWidth : integer;           //  
      i       : integer;
      bYValue     : integer;
      bXValue     : integer;
      bRect       : TRect;
      borderColor : TColor;
  begin
// ---------------------------------------------------------------------------
//                 
// ---------------------------------------------------------------------------
    bRect := pDrawVAxis;

    bRect.Left   := bRect.Left   + 2;
    bRect.Bottom := bRect.Bottom - 2;

    with bRect do
    begin
      bHeight := Bottom - Top;
      bWidth  := Right - Left;
    end;
// ---------------------------------------------------------------------------
//                               
// ---------------------------------------------------------------------------
     //   
    bXWidth := round(bWidth*fHistColWidth);

    fLBitmap.Canvas.Pen.Style := psSolid;
    fLBitmap.Canvas.Pen.Width := 1;

    for i:=0 to high (fValues) do
      if (fValues[i].x>=fMinX) and (fValues[i].x<=fMaxX) and
         (fValues[i].Y>=fMinY) and (fValues[i].y<=fMaxY) then
      begin
          //  YMax  
        bYValue := round((fValues[i].y-fMinY)/(fMaxY - fMinY)*bHeight);
          //     
        if bYValue>bHeight then bYValue := bHeight
        else if bYValue<0 then bYValue := 0;
          //  X  
        bXValue := round((fValues[i].x-fMinX)/(fMaxX - fMinX)*(bWidth-bXWidth));

          //  
        if fValuesBorderColor<>clNone then
             borderColor := fValuesBorderColor
        else borderColor := GetColorFromList(fValuesColor, bYValue/bHeight);

        case fGType of
          gtDot : with bRect do
                    iFillBox (fLBitmap,
                              Left+bXValue, Bottom - bYValue - round(bXWidth/2),
                              bXWidth+1,    bXWidth+1, borderColor, GetColorFromList(fValuesColor, bYValue/bHeight));
          gtRDot : with bRect do
                   begin
                     fLBitmap.Canvas.Brush.Color := GetColorFromList(fValuesColor, bYValue/bHeight);
                     fLBitmap.Canvas.Ellipse(Left+bXValue,           Bottom - bYValue - round(bXWidth/2),
                                             Left+bXValue+bXWidth+1, Bottom - bYValue - round(bXWidth/2)+bXWidth+1);
                   end;
        end;
      end;
  end;


  procedure TDJGraph.pDrawLinearHistogram;
  var bHeight, bWidth  : integer;  //   
      i           : integer;
      bYValue     : integer;
      bXValue     : integer;
      bRect       : TRect;
  begin
// ---------------------------------------------------------------------------
//                 
// ---------------------------------------------------------------------------
    bRect := pDrawVAxis;

    bRect.Left   := bRect.Left   + 2;
    bRect.Bottom := bRect.Bottom - 2;

    with bRect do
    begin
      bHeight := Bottom - Top;
      bWidth  := Right - Left;
    end;
// ---------------------------------------------------------------------------
//                               
// ---------------------------------------------------------------------------

    fLBitmap.Canvas.Pen.Assign (fPen);

    for i:=0 to high (fValues) do
      if (fValues[i].x>=fMinX) and (fValues[i].x<=fMaxX) then
      begin
          //  YMax  
        bYValue := round((fValues[i].y-fMinY)/(fMaxY - fMinY)*bHeight);
          //     
        if bYValue>bHeight then bYValue := bHeight
        else if bYValue<0 then bYValue := 0;
          //  X  
        bXValue := round((fValues[i].x-fMinX)/(fMaxX - fMinX)*bWidth);
          //  
        with bRect do
          if i=0 then
               fLBitmap.Canvas.MoveTo (Left + bXValue, Bottom - bYValue)
          else fLBitmap.Canvas.LineTo (Left + bXValue, Bottom - bYValue);
      end;
  end;


  procedure TDJGraph.pDrawDotLinearHistogram;
  var bHeight, bWidth  : integer;  //   
      bXWidth : integer;           //  
      i       : integer;
      bYValue     : integer;
      bXValue     : integer;
      bRect       : TRect;
      borderColor : TColor;
  begin
// ---------------------------------------------------------------------------
//                 
// ---------------------------------------------------------------------------
    bRect := pDrawVAxis;

    bRect.Left   := bRect.Left   + 2;
    bRect.Bottom := bRect.Bottom - 2;

    with bRect do
    begin
      bHeight := Bottom - Top;
      bWidth  := Right - Left;
    end;
// ---------------------------------------------------------------------------
//                               
// ---------------------------------------------------------------------------
     //   
    bXWidth := round(bWidth*fHistColWidth);
// ---------------------------------------------------------------------------
//                               
// ---------------------------------------------------------------------------

    fLBitmap.Canvas.Pen.Assign (fPen);

    for i:=0 to high (fValues) do
      if (fValues[i].x>=fMinX) and (fValues[i].x<=fMaxX) then
      begin
          //  YMax  
        bYValue := round((fValues[i].y-fMinY)/(fMaxY - fMinY)*bHeight);
          //     
        if bYValue>bHeight then bYValue := bHeight
        else if bYValue<0 then bYValue := 0;
          //  X  
        bXValue := round((fValues[i].x-fMinX)/(fMaxX - fMinX)*(bWidth-bXWidth)+bXWidth/2);
          //  
        with bRect do
          if i=0 then
               fLBitmap.Canvas.MoveTo (Left + bXValue, Bottom - bYValue)
          else fLBitmap.Canvas.LineTo (Left + bXValue, Bottom - bYValue);
      end;
// ---------------------------------------------------------------------------
//                               
// ---------------------------------------------------------------------------
    fLBitmap.Canvas.Pen.Style := psSolid;
    fLBitmap.Canvas.Pen.Width := 1;

    for i:=0 to high (fValues) do
      if (fValues[i].x>=fMinX) and (fValues[i].x<=fMaxX) then
      begin
          //  YMax  
        bYValue := round((fValues[i].y-fMinY)/(fMaxY - fMinY)*bHeight);
          //     
        if bYValue>bHeight then bYValue := bHeight
        else if bYValue<0 then bYValue := 0;
          //  X  
        bXValue := round((fValues[i].x-fMinX)/(fMaxX - fMinX)*(bWidth-bXWidth));

          //  
        if fValuesBorderColor<>clNone then
             borderColor := fValuesBorderColor
        else borderColor := GetColorFromList(fValuesColor, bYValue/bHeight);

        case fGType of
          gtLinearDot : with bRect do
                    iFillBox (fLBitmap,
                              Left+bXValue, Bottom - bYValue - round(bXWidth/2),
                              bXWidth+1,    bXWidth+1, borderColor, GetColorFromList(fValuesColor, bYValue/bHeight));
          gtLinearRDot : with bRect do
                   begin
                     fLBitmap.Canvas.Brush.Color := GetColorFromList(fValuesColor, bYValue/bHeight);
                     fLBitmap.Canvas.Ellipse(Left+bXValue,           Bottom - bYValue - round(bXWidth/2),
                                             Left+bXValue+bXWidth+1, Bottom - bYValue - round(bXWidth/2)+bXWidth+1);
                   end;
        end;
      end;
  end;


  procedure TDJGraph.pDrawHorizontalHistogram;
  var bHeight, bWidth  : integer;  //   
      bXWidth : integer;           //  
      i,nY,nX     : integer;
      bYValue     : integer;
      bXValue     : integer;
      BGR         : integer;
      bRect       : TRect;
  begin
// ---------------------------------------------------------------------------
//                 
// ---------------------------------------------------------------------------
    bRect := pDrawHAxis;

    bRect.Left   := bRect.Left   + 2;
    bRect.Bottom := bRect.Bottom - 2;

    with bRect do
    begin
      bHeight := Bottom - Top;
      bWidth  := Right - Left;
    end;
// ---------------------------------------------------------------------------
//                               
// ---------------------------------------------------------------------------
     //   
    bXWidth := round(bWidth*fHistColWidth);

    fLBitmap.Canvas.Pen.Style := psSolid;
    fLBitmap.Canvas.Pen.Width := 1;

    for i:=0 to high (fValues) do
      if (fValues[i].x>=fMinX) and (fValues[i].x<=fMaxX) then
      begin
          //  YMax(->X)  
        bYValue := round((fValues[i].y-fMinY)/(fMaxY - fMinY)*bWidth);
          //     
        if bYValue>bWidth then bYValue := bWidth
        else if bYValue<0 then bYValue := 0;
          //  X(->Y)  
        bXValue := round((fValues[i].x-fMinX)/(fMaxX - fMinX)*(bHeight-bXWidth));
          //  
        for nX := 1 to bYValue - 1 do
        begin
          BGR := Color2BGR (GetColorFromList(fValuesColor, nX/bWidth));
          for nY := 0 to bXWidth - 1 do
            with bRect do
              qPutPixel (fLBitmap, Left+nX, Bottom - nY - bXValue,BGR);
        end;

        if (fValuesBorderColor<>clNone) and (bYValue>0) then
           with bRect do
            iRectangle (fLBitmap,
                        Left,         Bottom-bXValue,
                        Left+bYValue, Bottom-bXValue-bXWidth, fValuesBorderColor);
      end;
  end;


//------------------------------------------------------------------------------
//                             PUBLIC
//------------------------------------------------------------------------------


  procedure TDJGraph.AddValue (iName: string; iX, iY : double);
  begin
    setlength (fValues, length(fValues)+1);
    with fValues [high(fValues)] do
    begin
      name := iName;
      x := ix;
      y := iy;
    end;
  end;


  procedure TDJGraph.ClearValues;
  begin
    setlength (fValues,0);
  end;


  procedure TDJGraph.Paint (Where : TBitmap; Area: TRect);
  const eps = 0.000001;
  var oldPen : TPen;
  begin

    if (abs(maxX-minX)<eps) or
       (abs(maxY-minY)<eps) then exit;
    fLBitmap  := Where;
    fDrawArea := Area;
    fLBitmap.pixelformat := pf32bit;
    oldPen := TPen.Create;
    oldPen.Assign (fLBitmap.Canvas.Pen);
    case fGType of
      gtDot        : pDrawDotHistogram;
      gtRDot       : pDrawDotHistogram;
      gtLinearDot  : pDrawDotLinearHistogram;
      gtLinearRDot : pDrawDotLinearHistogram;
      gtLinear     : pDrawLinearHistogram;
      gtHHistogram : pDrawHorizontalHistogram;
      gtVHistogram : pDrawVerticalHistogram;
    end;
    fLBitmap.Canvas.Pen.assign(oldPen);
    oldPen.free;
  end;

//-[ END ]---------------------------------------------------------------------
end.
