//***************************************************************************//
//*                                                                         *//
//* Unit:    drj_HTMLExporter                                               *//
//* Author:  Dark Magus                                                     *//
//* Date:    05.01.02                                                       *//
//*                                                                         *//
//*  ,       HTML   -  *//
//*       Dream Journal "Wizardry" (v. 2.0).         *//
//*                                                                         *//
//***************************************************************************//
unit drj_HTMLExporter;

//-[ INTERFACE ]---------------------------------------------------------------
interface

uses
  Sysutils,

  drj_Resources,
  drj_Exporter,

  xml_Node;

type THTMLExporter = class (TExporter)
  private
    tableNumber : Integer;

  protected
    function GetBody (iNode : TXMLNode) : String; override;
    function GetHeader (iNode : TXMLNode) : String; override;
    function GetFooter (iNode : TXMLNode) : String; override;
    function ExportLibrary (iNode : TXMLNode) : String;
    function ExportDay (iNode : TXMLNode) : String;
    function ExportDirectory (iNode : TXMLNode) : String;
    function ExportExperience (iNode : TXMLNode) : String;
    function ExportProperties (iNode : TXMLNode) : String;

  public
    constructor Create;
    destructor Destroy; override;

  published


end;

//-[ IMPLEMENTATION ]----------------------------------------------------------
implementation

constructor THTMLExporter.Create;
begin
  inherited Create;
  tableNumber := 1;
  SPACE := '&nbsp;';
end;

destructor THTMLExporter.Destroy;
begin
  inherited Destroy;
end;

function THTMLExporter.GetBody (iNode : TXMLNode) : String;
var
  i : Integer;
begin
  Result := '';
  if (iNode <> nil) then
  begin
    Result := Result + '<body>' + END_LINE;
    if (iNode.TagName = TAG_DREAMLIBRARY) then
    begin
      Result := Result + ExportLibrary (iNode);
    end;
    if (iNode.TagName = TAG_DREAMJOURNAL) then
    begin
      for i := 0 to iNode.GetNumberOfChildren - 1 do
      begin
        Result := Result + ExportDay (iNode.GetChild (i));
      end;
    end;
    Result := Result + '</body>' + END_LINE;
  end;
end;

function THTMLExporter.GetHeader (iNode : TXMLNode) : String;
var
  tempString : String;
begin
  Result := '';
  if (iNode <> nil) then
  begin
    Result := Result + '<html>' + END_LINE;
    Result := Result + '<head>' + END_LINE;
    Result := Result + '<title>';
    if (iNode.TagName = TAG_DREAMLIBRARY) then
    begin
      tempString := iNode.GetAttributeValue (ATTRIBUTE_NAME);
      if (tempString <> UNDEFINED_VALUE) then
      begin
        Result := Result + DREAM_LIBRARY_STRING + ': [' + tempString + ']</title>' + END_LINE;
      end
      else
      begin
        Result := Result + DREAM_LIBRARY_STRING;
      end;
      Result := Result + '<link rel="STYLESHEET" type="text/css" href="library.css">' + END_LINE;
    end;
    if (iNode.TagName = TAG_DREAMJOURNAL) then
    begin
      Result := Result + DREAM_JOURNAL_STRING + '</title>' + END_LINE;
      Result := Result + '<link rel="STYLESHEET" type="text/css" href="journal.css">' + END_LINE;
    end;
    Result := Result + '</head>' + END_LINE;
  end;
end;

function THTMLExporter.GetFooter (iNode : TXMLNode) : String;
begin
  Result := '';
  if (iNode <> nil) then
  begin
    Result := Result + '</html>';
  end;
end;

function THTMLExporter.ExportLibrary (iNode : TXMLNode) : String;
var
  i          : Integer;
  tempString : String;
  childNode  : TXMLNode;

  function formCells (iCell1, iCell2 : String) : String;
  begin
    Result := Result + '<tr>' + END_LINE;
    Result := Result + '<td class="userTableItem">' + END_LINE;
    Result := Result + iCell1 + END_LINE;
    Result := Result + '</td>' + END_LINE;
    Result := Result + '<td class="userTableValue">' + END_LINE;
    Result := Result + iCell2 + END_LINE;
    Result := Result + '</td>' + END_LINE;
    Result := Result + '</tr>' + END_LINE;
  end;

begin
  Result := '';
  tempString := iNode.GetAttributeValue (ATTRIBUTE_NAME);
  if (tempString <> UNDEFINED_VALUE) then
  begin
    Result := Result + '<h1>' + DREAM_LIBRARY_STRING + ': [' + tempString + ']</h1>' + END_LINE;
  end
  else
  begin
    Result := Result + '<h1>' + DREAM_LIBRARY_STRING + '</h1>' + END_LINE;
  end;

  childNode := iNode.FindTagByName (TAG_USER);
  if (childNode <> nil) then
  begin
    Result := Result + '<h4>' + TABLE_STRING + ' ' + IntToStr (tableNumber) + '. ' + USER_PROPERTIES_STRING + '</h4>' + END_LINE;
    tableNumber := tableNumber + 1;
    Result := Result + '<table class="userTable">' + END_LINE;

    tempString := childNode.GetAttributeValue (ATTRIBUTE_NAME);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + formCells (NAME_STRING, tempString);
    end;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_NICK);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + formCells (NICK_STRING, tempString);
    end;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_EMAIL);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + formCells (EMAIL_STRING, '<a href="mailto:' + tempString + '">' + tempString + '</a>');
    end;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_AGE);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + formCells (AGE_STRING, tempString);
    end;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_SEX);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      if (tempString = STR_MASCULINUM) then
      begin
        tempString := MASCULINUM_STRING;
      end;
      if (tempString = STR_FEMININUM) then
      begin
        tempString := FEMININUM_STRING;
      end;
      if (tempString = STR_SMILE) then
      begin
        tempString := SMILE_STRING;
      end;
      Result := Result + formCells (SEX_STRING, tempString);
    end;
    if (ChildNode.GetChild (0) <> nil) then
    begin
      Result := Result + formCells (ADDITIONAL_STRING, ChildNode.GetChild (0).TextData);
    end;

    Result := Result + '</table>' + END_LINE;
    Result := Result + '<br>' + END_LINE;
  end;

  Result := Result + '<h2>' + LIBRARY_STRUCTURE_STRING + '</h2>' + END_LINE;
  for i := 0 to iNode.GetNumberOfChildren - 1 do
  begin
    childNode := iNode.GetChild (i);
    if (childNode.TagName = TAG_DIRECTORY) then
    begin
      Result := Result + ExportDirectory (childNode);
    end;
  end;
  Result := Result + '<br>' + END_LINE;

  childNode := iNode.FindTagByName (TAG_ADDITIONAL);
  if (childNode <> nil) then
  begin
    Result := Result + '<h2>' + ADDITIONAL_FILES_STRING + '</h2>' + END_LINE;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_DICTIONARY);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + '<span class="additionalFileName">' + DICTIONARY_STRING + ': [' + SPACE + '</span>' + '<a href="' + tempString +  '" class="additionalFilePath">' + tempString + '</a>' + '<span class="additionalFileName">' + SPACE + ']</span>' + '<br>' + END_LINE;
    end;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_VOCABULARY);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + '<span class="additionalFileName">' + VOCABULARY_STRING + ': [' + SPACE + '</span>' + '<a href="' + tempString +  '" class="additionalFilePath">' + tempString + '</a>' + '<span class="additionalFileName">' + SPACE + ']</span>' + '<br>' + END_LINE;
    end;
  end;
end;

function THTMLExporter.ExportDirectory (iNode : TXMLNode) : String;
var
  i         : Integer;
  childNode : TXMLNode;
begin
  Result := '<table class="structureTable">' + END_LINE;
  Result := Result + '<tr>' + END_LINE;
  Result := Result + '<td colspan="2" class="structureDirectoryName">' + END_LINE;
  Result := Result + iNode.GetAttributeValue (ATTRIBUTE_NAME) + END_LINE;
  Result := Result + '</td>' + END_LINE;
  Result := Result + '</tr>' + END_LINE;
  Result := Result + '<tr>' + END_LINE;
  Result := Result + '<td class="structureChilds">' + SPACE + '<br></td>' + END_LINE;
  Result := Result + '<td class="structureChilds">' + END_LINE;
  for i := 0 to iNode.GetNumberOfChildren - 1 do
  begin
    childNode := iNode.GetChild (i);
    if (childNode.TagName = TAG_DIRECTORY) then
    begin
      Result := Result + ExportDirectory (childNode);
    end
    else
    begin
      if (childNode.TagName = TAG_FILE) then
      begin
        Result := Result + '<span class="fileName">' + childNode.GetAttributeValue (ATTRIBUTE_NAME) + ': [' + SPACE + '</span>' + '<a href="' + childNode.GetAttributeValue (ATTRIBUTE_PATH) + '">'+ childNode.GetAttributeValue (ATTRIBUTE_PATH) + '</a>' + '<span class="fileName">' + SPACE + ']' + '</span>' + '<br>' + END_LINE;
      end;
    end;
  end;
  Result := Result + '</td>' + END_LINE;
  Result := Result + '</tr>' + END_LINE;
  Result := Result + '</table>' + END_LINE;
end;

function THTMLExporter.ExportDay (iNode : TXMLNode) : String;
var
  i          : Integer;
  tempString : String;
  childNode  : TXMLNode;
begin
  Result := '';
  if ((iNode <> nil) and (iNode.TagName = TAG_DAY)) then
  begin
    tempString := iNode.GetAttributeValue (ATTRIBUTE_NAME);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + '<h1>' + tempString + '</h1>' + END_LINE;
    end;
    for i := 0 to iNode.GetNumberOfChildren - 1 do
    begin
      childNode := iNode.GetChild (i);
      if (childNode.TagName = TAG_EXPERIENCE) then
      begin
        Result := Result + ExportExperience (childNode);
      end;
      if ((childNode.TagName = TAG_COMMENT) and (childNode.GetChild (0) <> nil)) then
      begin
        Result := Result + '<h3>' + DAY_COMMENT_STRING + '</h3>' + END_LINE;
        Result := Result + '<span class="dayCommentBody">' + childNode.GetChild(0).TextData + '</span>' + END_LINE;
      end;
    end;
  end;
end;

function THTMLExporter.ExportExperience (iNode : TXMLNode) : String;
var
  tempReal   : Real;
  tempString : String;
  childNode  : TXMLNode;
begin
  Result := '<h2>';
  tempString := iNode.GetAttributeValue (ATTRIBUTE_NUMBER);
  if (tempString <> UNDEFINED_VALUE) then
  begin
    Result := Result + tempString;
  end;
  Result := Result + NAME_DELIMITER;
  tempString := iNode.GetAttributeValue (ATTRIBUTE_TITLE);
  if (tempString <> UNDEFINED_VALUE) then
  begin
    Result := Result + tempString;
  end;
  Result := Result + '</h2>' + END_LINE;

  childNode := iNode.FindTagByName (TAG_DESCRIPTION);
  if ((childNode <> nil) and (childNode.GetChild (0) <> nil)) then
  begin
    Result := Result + '<span class="dreamDescription">' + childNode.GetChild (0).TextData + '</span>' + END_LINE;
  end;

  childNode := iNode.FindTagByName (TAG_COMMENT);
  if ((childNode <> nil) and (childNode.GetChild (0) <> nil)) then
  begin
    Result := Result + '<h3>' + EXPERIENCE_COMMENT_STRING + '</h3>' + END_LINE;
    Result := Result + '<span class="dreamCommentBody">' + childNode.GetChild (0).TextData + '</span>' + END_LINE;
  end;

  childNode := iNode.FindTagByName (TAG_ENERGY);
  if (childNode <> nil) then
  begin
    tempReal := 0.0;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_LEVEL);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      tempReal := (100.0 - StrToFloat (tempString)) / 100.0;
    end;
    Result := Result + '<h4>' + ENERGY_LEVEL_STRING + FloatToStr (tempReal) + '</h4>' + END_LINE;
  end;

  childNode := iNode.FindTagByName (TAG_IMAGE);
  if (childNode <> nil) then
  begin
    tempString := childNode.GetAttributeValue (ATTRIBUTE_PATH);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + '<table class="imageTable">' + END_LINE;
      Result := Result + '<tr>' + END_LINE;
      Result := Result + '<td class="imageTitle">' + END_LINE;
      Result := Result + '<h4>' + IMAGE_STRING + '</h4>' + END_LINE;
      Result := Result + '</td>' + END_LINE;
      Result := Result + '<td class="imageImage">' + END_LINE;
      Result := Result + '<img src="' + tempString + '">' + END_LINE;
      Result := Result + '</td>' + END_LINE;
      Result := Result + '</tr>' + END_LINE;
      Result := Result + '</table>' + END_LINE;
    end;
  end;

  childNode := iNode.FindTagByName (TAG_PROPERTIES);
  Result := Result + '<h4>' + PROPERTIES_STRING + '</h4>' + END_LINE;
  Result := Result + '<table class="propertiesTable">' + END_LINE;
  Result := Result + '<tr class="propertiesTableHeader">' + END_LINE;
  Result := Result + '<td>' + PROPERTY_STRING + '</td>' + END_LINE;
  Result := Result + '<td>' + VALUE_STRING + '</td>' + END_LINE;
  Result := Result + '</tr>' + END_LINE;
  if (childNode <> nil) then
  begin
    tempString := childNode.GetAttributeValue (ATTRIBUTE_LABEL1);
    Result := Result + '<tr>' + END_LINE;
    Result := Result + '<td class="propertiesTableValueName">' + Flag_1 + ':' + '</td>' + END_LINE;
    if (tempString = TRUE_VALUE) then
    begin
      Result := Result + '<td class="propertiesTableValue">' + YES_STRING + '</td>' + END_LINE;
    end
    else
    begin
      Result := Result + '<td class="propertiesTableValue">' + NO_STRING + '</td>' + END_LINE;
    end;
    Result := Result + '</tr>' + END_LINE;

    tempString := childNode.GetAttributeValue (ATTRIBUTE_LABEL2);
    Result := Result + '<tr>' + END_LINE;
    Result := Result + '<td class="propertiesTableValueName">' + Flag_2 + ':' + '</td>' + END_LINE;
    if (tempString = TRUE_VALUE) then
    begin
      Result := Result + '<td class="propertiesTableValue">' + YES_STRING + '</td>' + END_LINE;
    end
    else
    begin
      Result := Result + '<td class="propertiesTableValue">' + NO_STRING + '</td>' + END_LINE;
    end;
    Result := Result + '</tr>' + END_LINE;

    tempString := childNode.GetAttributeValue (ATTRIBUTE_LABEL3);
    Result := Result + '<tr>' + END_LINE;
    Result := Result + '<td class="propertiesTableValueName">' + Flag_3 + ':' + '</td>' + END_LINE;
    if (tempString = TRUE_VALUE) then
    begin
      Result := Result + '<td class="propertiesTableValue">' + YES_STRING + '</td>' + END_LINE;
    end
    else
    begin
      Result := Result + '<td class="propertiesTableValue">' + NO_STRING + '</td>' + END_LINE;
    end;
    Result := Result + '</tr>' + END_LINE;

    tempString := childNode.GetAttributeValue (ATTRIBUTE_LABEL4);
    Result := Result + '<tr>' + END_LINE;
    Result := Result + '<td class="propertiesTableValueName">' + Flag_4 + ':' + '</td>' + END_LINE;
    if (tempString = TRUE_VALUE) then
    begin
      Result := Result + '<td class="propertiesTableValue">' + YES_STRING + '</td>' + END_LINE;
    end
    else
    begin
      Result := Result + '<td class="propertiesTableValue">' + NO_STRING + '</td>' + END_LINE;
    end;
    Result := Result + '</tr>' + END_LINE;

    Result := Result + '</table>' + END_LINE;

    if (childNode.GetNumberOfChildren > 0) then
    begin
      Result := Result + '<h4>' + ADDITIONAL_PROPERTIES_STRING + '</h4>' + END_LINE;
      Result := Result + '<table class="propertiesTable">' + END_LINE;
      Result := Result + '<tr class="propertiesTableHeader">' + END_LINE;
      Result := Result + '<td>' + PROPERTY_STRING + '</td>' + END_LINE;
      Result := Result + '<td>' + VALUE_STRING + '</td>' + END_LINE;
      Result := Result + '</tr>' + END_LINE;
      Result := Result + ExportProperties (childNode);
      Result := Result + '</table>' + END_LINE;
    end
    else
    begin
      Result := Result + '<h4>' + ADDITIONAL_PROPERTIES_STRING + NO_PROPERTIES_STRING + '</h4>' + END_LINE;
    end;
  end
  else
  begin
    Result := Result + '<tr>' + END_LINE;
    Result := Result + '<td class="propertiesTableValueName">' + Flag_1 + ':' + '</td>' + END_LINE;
    Result := Result + '<td class="propertiesTableValue">' + NO_STRING + '</td>' + END_LINE;
    Result := Result + '</tr>' + END_LINE;
    Result := Result + '<tr>' + END_LINE;
    Result := Result + '<td class="propertiesTableValueName">' + Flag_2 + ':' + '</td>' + END_LINE;
    Result := Result + '<td class="propertiesTableValue">' + NO_STRING + '</td>' + END_LINE;
    Result := Result + '</tr>' + END_LINE;
    Result := Result + '<tr>' + END_LINE;
    Result := Result + '<td class="propertiesTableValueName">' + Flag_3 + ':' + '</td>' + END_LINE;
    Result := Result + '<td class="propertiesTableValue">' + NO_STRING + '</td>' + END_LINE;
    Result := Result + '</tr>' + END_LINE;
    Result := Result + '<tr>' + END_LINE;
    Result := Result + '<td class="propertiesTableValueName">' + Flag_4 + ':' + '</td>' + END_LINE;
    Result := Result + '<td class="propertiesTableValue">' + NO_STRING + '</td>' + END_LINE;
    Result := Result + '</tr>' + END_LINE;
    Result := Result + '</table>' + END_LINE;
    Result := Result + '<h4>' + ADDITIONAL_PROPERTIES_STRING + NO_PROPERTIES_STRING + '</h4>' + END_LINE;
  end;
end;

function THTMLExporter.ExportProperties (iNode : TXMLNode) : String;
var
  i          : Integer;
  childNode  : TXMLNode;
begin
  Result := '';
  for i := 0 to iNode.GetNumberOfChildren - 1 do
  begin
    childNode := iNode.GetChild (i);
    if (childNode.GetNumberOfChildren > 0) then
    begin
      Result := Result + ExportProperties (childNode);
    end
    else
    begin
      Result := Result + '<tr>' + END_LINE;
      Result := Result + '<td class="propertiesTableValueName">' + childNode.GetAttributeValue (ATTRIBUTE_CONTROL) + ': </td><td class="propertiesTableValue">' + childNode.GetAttributeValue (ATTRIBUTE_VALUE) + '</td>' + END_LINE;
      Result := Result + '</tr>' + END_LINE;
    end;
  end;
end;

//-[ END ]---------------------------------------------------------------------
end.
