//***************************************************************************//
//*                                                                         *//
//* Unit:    drj_TXTExporter                                                *//
//* Author:  Dark Magus                                                     *//
//* Date:    05.01.02                                                       *//
//*                                                                         *//
//*  ,       TXT   -   *//
//*       Dream Journal "Wizardry" (v. 2.0).         *//
//*                                                                         *//
//***************************************************************************//
unit drj_TXTExporter;

//-[ INTERFACE ]---------------------------------------------------------------
interface

uses
  Sysutils,

  drj_Resources,
  drj_Exporter,

  xml_Node;

type TTXTExporter = class (TExporter)
  protected
    DASH      : String;

    function GetBody (iNode : TXMLNode) : String; override;
    function GetHeader (iNode : TXMLNode) : String; override;
    function GetFooter (iNode : TXMLNode) : String; override;
    function ExportDay (iNode : TXMLNode) : String;
    function ExportProperties (iNode : TXMLNode) : String;
    function ExportLibrary (iNode : TXMLNode) : String;
    function ExportDirectory (iNode : TXMLNode) : String;
    function GetNDashes (N : Integer) : String;

  public
    constructor Create;
    destructor Destroy; override;

  published


end;

//-[ IMPLEMENTATION ]----------------------------------------------------------
implementation

constructor TTXTExporter.Create;
begin
  inherited Create;
  SEPARATOR := '+------------------------------------------------------------------------------+';
  SPACE := ' ';
  DASH := '-';
end;

destructor TTXTExporter.Destroy;
begin
  inherited Destroy;
end;

function TTXTExporter.GetNDashes (N : Integer) : String;
var
  i : Integer;
begin
  Result := '';
  if (N > 0) then
  begin
    for i := 1 to N do
    begin
      Result := Result + DASH;
    end;
  end;
end;

function TTXTExporter.GetBody (iNode : TXMLNode) : String;
var
  i : Integer;
begin
  Result := '';
  if (iNode <> nil) then
  begin
    if (iNode.TagName = TAG_DREAMLIBRARY) then
    begin
      Result := Result + ExportLibrary (iNode);
      Exit;
    end;
    if (iNode.TagName = TAG_DREAMJOURNAL) then
    begin
      for i := 0 to iNode.GetNumberOfChildren - 1 do
      begin
        Result := Result + ExportDay (iNode.GetChild (i));
      end;
      Exit;
    end;
  end;
end;

function TTXTExporter.GetHeader (iNode : TXMLNode) : String;
var
  tempInt    : Integer;
  tempString : String;
  header     : String;
begin
  Result := '';
  tempString := '';
  header := '';
  if (iNode <> nil) then
  begin
    if (iNode.TagName = TAG_DREAMLIBRARY) then
    begin
      tempString := iNode.GetAttributeValue (ATTRIBUTE_NAME);
      if (tempString <> UNDEFINED_VALUE) then
      begin
        header := DREAM_LIBRARY_STRING + ': [' + tempString + ']';
      end
      else
      begin
        header := DREAM_LIBRARY_STRING;
      end;
      tempInt := (78 - Length (header)) div 2;
      tempString := '|' + GetNSpaces (tempInt) + header + GetNSpaces (tempInt);
      if (tempInt * 2 + 2 + Length (header) < 80) then
      begin
        tempString := tempString + SPACE;
      end;
      tempString := tempString + '|';
      Result := Result + SEPARATOR + END_LINE;
      Result := Result + tempString + END_LINE;
      Result := Result + SEPARATOR + END_LINE;
      Result := Result + END_LINE;
      Exit;
    end;
    if (iNode.TagName = TAG_DREAMJOURNAL) then
    begin
      tempInt := (78 - Length (DREAM_JOURNAL_STRING)) div 2;
      tempString := '|' + GetNSpaces (tempInt) + DREAM_JOURNAL_STRING + GetNSpaces (tempInt);
      if (tempInt * 2 + 2 + Length (DREAM_JOURNAL_STRING) < 80) then
      begin
        tempString := tempString + SPACE;
      end;
      tempString := tempString + '|';
      Result := Result + SEPARATOR + END_LINE;
      Result := Result + tempString + END_LINE;
      Result := Result + SEPARATOR + END_LINE;
      Result := Result + END_LINE;
      Exit;
    end;
  end;
end;

function TTXTExporter.GetFooter (iNode : TXMLNode) : String;
begin
  Result := '';
  if (iNode <> nil) then
  begin
    Result := Result + SEPARATOR;
  end;
end;

function TTXTExporter.ExportDay (iNode : TXMLNode) : String;
var
  i           : Integer;
  tempReal    : Real;
  tempString  : String;
  otherString : String;
  childNode   : TXMLNode;
  nextNode    : TXMLNode;
begin
  Result := '';
  if (iNode <> nil) then
  begin
    if (iNode.TagName = TAG_DAY) then
    begin
      tempString := iNode.GetAttributeValue (ATTRIBUTE_NAME);
      if (tempString <> UNDEFINED_VALUE) then
      begin
        otherString := '    +' + GetNDashes (Length (tempString) + 4) + '+' + END_LINE;
        Result := Result + otherString;
        Result := Result + '+' + GetNDashes (3) + '|  ' + tempString + '  |' + GetNDashes (69 - Length (tempString)) + '+' + END_LINE;
        Result := Result + otherString;
      end;
      Result := Result + END_LINE;
      for i := 0 to iNode.GetNumberOfChildren - 1 do
      begin
        childNode := iNode.GetChild (i);
        if (childNode.TagName = TAG_EXPERIENCE) then
        begin
          Result := Result + childNode.GetAttributeValue (ATTRIBUTE_NUMBER) + NAME_DELIMITER + childNode.GetAttributeValue (ATTRIBUTE_TITLE) + END_LINE;
          nextNode := childNode.FindTagByName (TAG_DESCRIPTION);
          if (nextNode <> nil) then
          begin
            if (nextNode.GetChild (0) <> nil) then
            begin
              Result := Result + nextNode.GetChild (0).TextData + END_LINE;
            end;
          end;
          nextNode := childNode.FindTagByName (TAG_COMMENT);
          if (nextNode <> nil) then
          begin
            Result := Result + EXPERIENCE_COMMENT_STRING;
            if (nextNode.GetChild (0) <> nil) then
            begin
              Result := Result + nextNode.GetChild (0).TextData;
            end;
            Result := Result + END_LINE;
          end;
          nextNode := childNode.FindTagByName (TAG_ENERGY);
          tempReal := 0.0;
          if (nextNode <> nil) then
          begin
            tempString := nextNode.GetAttributeValue (ATTRIBUTE_LEVEL);
            if (tempString <> UNDEFINED_VALUE) then
            begin
              tempReal := (100.0 - StrToFloat (tempString)) / 100.0;
            end;
          end;
          Result := Result + ENERGY_LEVEL_STRING + FloatToStr (tempReal) + END_LINE;
          nextNode := childNode.FindTagByName (TAG_IMAGE);
          if (nextNode <> nil) then
          begin
            Result := Result + IMAGE_STRING;
            tempString := nextNode.GetAttributeValue (ATTRIBUTE_PATH);
            if (tempString <> UNDEFINED_VALUE) then
            begin
              Result := Result + tempString;
            end;
            Result := Result + END_LINE;
          end;
          nextNode := childNode.FindTagByName (TAG_PROPERTIES);
          Result := Result + PROPERTIES_STRING + END_LINE;
          if (nextNode <> nil) then
          begin
            tempString := nextNode.GetAttributeValue (ATTRIBUTE_LABEL1);
            if (tempString = TRUE_VALUE) then
            begin
              Result := Result + GetNSpaces (4) + Flag_1 + ': ' + YES_STRING + END_LINE;
            end
            else
            begin
              Result := Result + GetNSpaces (4) + Flag_1 + ': ' + NO_STRING + END_LINE;
            end;
            tempString := nextNode.GetAttributeValue (ATTRIBUTE_LABEL2);
            if (tempString = TRUE_VALUE) then
            begin
              Result := Result + GetNSpaces (4) + Flag_2 + ': ' + YES_STRING + END_LINE;
            end
            else
            begin
              Result := Result + GetNSpaces (4) + Flag_2 + ': ' + NO_STRING + END_LINE;
            end;
            tempString := nextNode.GetAttributeValue (ATTRIBUTE_LABEL3);
            if (tempString = TRUE_VALUE) then
            begin
              Result := Result + GetNSpaces (4) + Flag_3 + ': ' + YES_STRING + END_LINE;
            end
            else
            begin
              Result := Result + GetNSpaces (4) + Flag_3 + ': ' + NO_STRING + END_LINE;
            end;
            tempString := nextNode.GetAttributeValue (ATTRIBUTE_LABEL4);
            if (tempString = TRUE_VALUE) then
            begin
              Result := Result + GetNSpaces (4) + Flag_4 + ': ' + YES_STRING + END_LINE;
            end
            else
            begin
              Result := Result + GetNSpaces (4) + Flag_4 + ': ' + NO_STRING + END_LINE;
            end;
            Result := Result + ADDITIONAL_PROPERTIES_STRING + END_LINE;
            Result := Result + ExportProperties (nextNode);
            if (nextNode.GetNumberOfChildren = 0) then
            begin
              Result := Result + GetNSpaces (4) + NO_PROPERTIES_STRING + END_LINE;
            end;
          end
          else
          begin
            Result := Result + GetNSpaces (4) + Flag_1 + ': ' + NO_STRING + END_LINE;
            Result := Result + GetNSpaces (4) + Flag_2 + ': ' + NO_STRING + END_LINE;
            Result := Result + GetNSpaces (4) + Flag_3 + ': ' + NO_STRING + END_LINE;
            Result := Result + GetNSpaces (4) + Flag_4 + ': ' + NO_STRING + END_LINE;
            Result := Result + ADDITIONAL_PROPERTIES_STRING + END_LINE;
            Result := Result + GetNSpaces (4) + NO_PROPERTIES_STRING + END_LINE;
          end;
          Result := Result + END_LINE;
        end;
      end;
      childNode := iNode.FindTagByName (TAG_COMMENT);
      if ((childNode <> nil) and (iNode.GetChildNumber (childNode) <> -1)) then
      begin
        Result := Result + DAY_COMMENT_STRING;
        if (childNode.GetChild (0) <> nil) then
        begin
          Result := Result + childNode.GetChild (0).TextData + END_LINE;
        end;
        Result := Result + END_LINE;
      end;
    end;
  end;
end;

function TTXTExporter.ExportProperties (iNode : TXMLNode) : String;
var
  i          : Integer;
  childNode  : TXMLNode;
begin
  Result := '';
  if (iNode.GetNumberOfChildren > 0) then
  begin
    for i := 0 to iNode.GetNumberOfChildren - 1 do
    begin
      childNode := iNode.GetChild (i);
      if (childNode.GetNumberOfChildren > 0) then
      begin
        Result := Result + ExportProperties (childNode);
      end
      else
      begin
        Result := Result + GetNSpaces (4) + childNode.GetAttributeValue (ATTRIBUTE_CONTROL) + ': ' + childNode.GetAttributeValue (ATTRIBUTE_VALUE) + END_LINE;
      end;
    end;
  end;
end;

function TTXTExporter.ExportLibrary (iNode : TXMLNode) : String;
var
  i          : Integer;
  tempString : String;
  childNode  : TXMLNode;
begin
  Result := '';
  childNode := iNode.FindTagByName (TAG_USER);
  if (childNode <> nil) then
  begin
    Result := Result + USER_STRING + END_LINE;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_NAME);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + GetNSpaces (2) + NAME_STRING + tempString + END_LINE;
    end;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_NICK);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + GetNSpaces (2) + NICK_STRING + tempString + END_LINE;
    end;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_EMAIL);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + GetNSpaces (2) + EMAIL_STRING + tempString + END_LINE;
    end;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_AGE);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      Result := Result + GetNSpaces (2) + AGE_STRING + tempString + END_LINE;
    end;
    tempString := childNode.GetAttributeValue (ATTRIBUTE_SEX);
    if (tempString <> UNDEFINED_VALUE) then
    begin
      if (tempString = STR_MASCULINUM) then
      begin
        Result := Result + GetNSpaces (2) + SEX_STRING + MASCULINUM_STRING + END_LINE;
      end;
      if (tempString = STR_FEMININUM) then
      begin
        Result := Result + GetNSpaces (2) + SEX_STRING + FEMININUM_STRING + END_LINE;
      end;
      if (tempString = STR_SMILE) then
      begin
        Result := Result + GetNSpaces (2) + SEX_STRING + SMILE_STRING + END_LINE;
      end;
    end;
    if (ChildNode.GetChild (0) <> nil) then
    begin
      Result := Result + GetNSpaces (2) + ADDITIONAL_STRING + ChildNode.GetChild (0).TextData + END_LINE;
    end;

  end;

  Result := Result + END_LINE + FILELIST_STRING + END_LINE;
  for i := 0 to iNode.GetNumberOfChildren - 1 do
  begin
    childNode := iNode.GetChild (i);
    if (childNode.TagName = TAG_DIRECTORY) then
    begin
      Result := Result + ExportDirectory (childNode);
    end;
  end;
end;

function TTXTExporter.ExportDirectory (iNode : TXMLNode) : String;
var
  i         : Integer;
  childNode : TXMLNode;
begin
  Result := '';
  for i := 0 to iNode.GetNumberOfChildren - 1 do
  begin
    childNode := iNode.GetChild (i);
    if (childNode.TagName = TAG_DIRECTORY) then
    begin
      Result := Result + ExportDirectory (childNode);
    end
    else
    begin
      if (childNode.TagName = TAG_FILE) then
      begin
        Result := Result + GetNSpaces (2) + childNode.GetAttributeValue (ATTRIBUTE_NAME) + ': [' + childNode.GetAttributeValue (ATTRIBUTE_PATH) + ']' + END_LINE;
      end;
    end;
  end;
end;

//-[ END ]---------------------------------------------------------------------
end.
