//***************************************************************************//
//*                                                                         *//
//* Unit:    drj_XMLPairList                                                *//
//* Author:  Dark Magus                                                     *//
//* Date:    04.09.01                                                       *//
//*                                                                         *//
//*               *//
//* TXMLNode   Dream Journal (v. 2.0).                              *//
//*                                                                         *//
//***************************************************************************//
unit drj_XMLPairList;

//-[ INTERFACE ]---------------------------------------------------------------
interface

uses
  Classes,

  drj_Types,

  xml_node;

type TXMLPairList = class (TObject)
  private
    FPairList : TList;

  public
    constructor Create;
    destructor Destroy; override;

    procedure AddPair (fileNode, journalNode : TXMLNode);
    procedure AddOneInstance (fileNode, journalNode : TXMLNode);

    function FindFileByJournal (journalNode : TXMLNode) : TXMLNode;
    function FindJournalByFile (fileNode : TXMLNode) : TXMLNode;

end;

//-[ IMPLEMENTATION ]----------------------------------------------------------
implementation

constructor TXMLPairList.Create;
begin
  inherited Create;
  FPairList := TList.Create;
end;

destructor TXMLPairList.Destroy;
var
  i        : Integer;
  tempPair : PXMLPair;
begin
  for i := 0 to FPairList.Count - 1 do
  begin
    tempPair := FPairList.Items[i];
    Dispose (tempPair);
  end;
  FPairList.Clear;
  FPairList.Free;
  inherited Destroy;
end;

procedure TXMLPairList.AddPair (fileNode, journalNode : TXMLNode);
var
  tempPair : PXMLPair;
begin
  New (tempPair);
  tempPair.fileNode := fileNode;
  tempPair.journalNode := journalNode;
  FPairList.Add (tempPair);
end;

procedure TXMLPairList.AddOneInstance (fileNode, journalNode : TXMLNode);
var
  i        : Integer;
  tempPair : PXMLPair;
begin
  for i := 0 to FPairList.Count - 1 do
  begin
    tempPair := FPairList.Items[i];
    if ((tempPair.fileNode = fileNode) and (tempPair.journalNode = journalNode)) then
    begin
      Exit;
    end;
  end;
  New (tempPair);
  tempPair.fileNode := fileNode;
  tempPair.journalNode := journalNode;
  FPairList.Add (tempPair);
end;

function TXMLPairList.FindFileByJournal (journalNode : TXMLNode) : TXMLNode;
var
  i        : Integer;
  tempPair : PXMLPair;
begin
  Result := nil;
  for i := 0 to FPairList.Count - 1 do
  begin
    tempPair := FPairList.Items[i];
    if (tempPair.journalNode = journalNode) then
    begin
      Result := tempPair.fileNode;
      Exit;
    end;
  end;
end;

function TXMLPairList.FindJournalByFile (fileNode : TXMLNode) : TXMLNode;
var
  i        : Integer;
  tempPair : PXMLPair;
begin
  Result := nil;
  for i := 0 to FPairList.Count - 1 do
  begin
    tempPair := FPairList.Items[i];
    if (tempPair.fileNode = fileNode) then
    begin
      Result := tempPair.journalNode;
      Exit;
    end;
  end;
end;

//-[ END ]---------------------------------------------------------------------
end.
